"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcWebSocketServer = exports.RpcTcpServer = void 0;
const core_1 = require("@deepkit/core");
const fs_1 = require("fs");
const net_1 = require("net");
const ws_1 = require("ws");
/**
 * Uses the node `net` module to create a server. Supports unix sockets.
 */
class RpcTcpServer {
    constructor(kernel, host) {
        this.kernel = kernel;
        this.host = (0, core_1.parseHost)(host);
        if (this.host.isUnixSocket && (0, fs_1.existsSync)(this.host.unixSocket)) {
            (0, fs_1.unlinkSync)(this.host.unixSocket);
        }
    }
    start() {
        return (0, core_1.asyncOperation)((resolve, reject) => {
            this.server = (0, net_1.createServer)();
            this.server.on('listening', () => {
                resolve(true);
            });
            this.server.on('error', (err) => {
                reject(new Error('Could not start broker server: ' + err));
            });
            this.server.on('connection', (socket) => {
                var _a;
                const connection = (_a = this.kernel) === null || _a === void 0 ? void 0 : _a.createConnection({
                    write: function write(b) {
                        socket.write(b);
                    },
                    clientAddress: function clientAddress() {
                        return socket.remoteAddress || '';
                    },
                    close: function close() {
                        socket.destroy();
                    },
                    bufferedAmount: function bufferedAmount() {
                        return socket.writableLength || 0;
                    }
                });
                socket.on('data', (data) => {
                    connection.feed(data);
                });
                socket.on('close', () => {
                    connection.close();
                });
                socket.on('error', () => {
                    connection.close();
                });
            });
            if (this.host.isUnixSocket) {
                this.server.listen(this.host.unixSocket);
            }
            else {
                this.server.listen(this.host.port || 8811, this.host.host);
            }
        });
    }
    close() {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.close();
    }
}
exports.RpcTcpServer = RpcTcpServer;
class RpcWebSocketServer {
    constructor(kernel, host) {
        this.kernel = kernel;
        this.host = (0, core_1.parseHost)(host);
        if (this.host.isUnixSocket && (0, fs_1.existsSync)(this.host.unixSocket)) {
            (0, fs_1.unlinkSync)(this.host.unixSocket);
        }
    }
    close() {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.close();
    }
    start(options) {
        const defaultOptions = { host: this.host.host, port: this.host.port };
        this.server = new ws_1.WebSocketServer({ ...defaultOptions, ...options });
        this.server.on('connection', (ws, req) => {
            var _a;
            const connection = (_a = this.kernel) === null || _a === void 0 ? void 0 : _a.createConnection({
                write: function write(b) {
                    ws.send(b);
                },
                close: function close() {
                    ws.close();
                },
                bufferedAmount: function bufferedAmount() {
                    return ws.bufferedAmount;
                },
                clientAddress: function clientAddress() {
                    return req.socket.remoteAddress || '';
                }
            });
            ws.on('message', async (message) => {
                connection.feed(message);
            });
            ws.on('close', async () => {
                connection.close();
            });
        });
    }
}
exports.RpcWebSocketServer = RpcWebSocketServer;
//# sourceMappingURL=server.js.map